/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "MixCastOpaqueCanvas.h"

#include "MixCastDefines.h"

#include "Engine/Canvas.h"
#include "UObject/ConstructorHelpers.h"

UMixCastOpaqueCanvas::UMixCastOpaqueCanvas(const FObjectInitializer& ObjectInitializer)
	: Super(ObjectInitializer)
{
#if UE_4_22_OR_LATER
	bNeedsTwoCopies = true;
#endif

	static ConstructorHelpers::FObjectFinder<UMaterial> Find_CombineOpaqueMat(TEXT("Material'/MixCast/TransferWithSeparateFlippedAlphaChannel.TransferWithSeparateFlippedAlphaChannel'"));

	if (Find_CombineOpaqueMat.Succeeded())
	{
		combineMat = UMaterialInstanceDynamic::Create(
			Cast<UMaterialInterface>(Find_CombineOpaqueMat.Object),
			this,
			TEXT("Combine_Opaque_Layers")
			);
	}

	OnCanvasRenderTargetUpdate.AddUniqueDynamic(this, &UMixCastOpaqueCanvas::MergeTargets);
}

void UMixCastOpaqueCanvas::Initialize(UTextureRenderTarget2D* HdrOpaqueTex, UTextureRenderTarget2D* FinalOpaqueTex)
{
	ClearColor = FColor::Black;
	bAutoGenerateMips = false;
	TargetGamma = 2.2f;
	InitCustomFormat(HdrOpaqueTex->SizeX, HdrOpaqueTex->SizeY, PF_FloatRGBA, true);
	UpdateResourceImmediate();

	combineMat->SetTextureParameterValue(TEXT("AlphaTex"), HdrOpaqueTex);
	combineMat->SetTextureParameterValue(TEXT("MainTex"), FinalOpaqueTex);
}

void UMixCastOpaqueCanvas::MergeTargets(UCanvas* Canvas, int32 Width, int32 Height)
{
	Canvas->K2_DrawMaterial(
		combineMat,
		FVector2D::ZeroVector,
		FVector2D(Width, Height),
		FVector2D::ZeroVector
		);
}