/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "CoreMinimal.h"
#include "UObject/WeakObjectPtr.h"
#include "Components/SceneCaptureComponent2D.h"
#include "RendererInterface.h"
#include "SceneViewExtension.h"

#include "ExpPipelineTypes.h"
#include "Transmission/Unreal-Interop/SharedTextures/WatchedUnrealSharedTexture.h"
#include "MixCastShaders.h"

#include "MixCastSceneCaptureComponent2D.generated.h"

UCLASS()
class UMixCastSceneCaptureComponent2D : public USceneCaptureComponent2D
{
	GENERATED_BODY()

public:	
	DECLARE_EVENT_OneParam(UMixCastSceneCaptureComponent2D, FCaptureEvent, UMixCastSceneCaptureComponent2D*)

private:
#if MIXCAST_SHADER_PASSES
	class FMixCastSceneViewExt : public ISceneViewExtension
	{
	public:
		UMixCastSceneCaptureComponent2D* Parent;

		UTexture2D* OcclusionTex = NULL;

		void SetupViewFamily(FSceneViewFamily& InViewFamily) override {};
		void SetupView(FSceneViewFamily& InViewFamily, FSceneView& InView) override {};
		void BeginRenderViewFamily(FSceneViewFamily& InViewFamily) override {};

		void PreRenderViewFamily_RenderThread(FRHICommandListImmediate& RHICmdList, FSceneViewFamily& InViewFamily) override {};
		void PreRenderView_RenderThread(FRHICommandListImmediate& RHICmdList, FSceneView& InView) override {};

		void PostRenderBasePass_RenderThread(FRHICommandListImmediate& RHICmdList, FSceneView& InView) override;
	};
#endif

	FCaptureEvent CaptureBegin, CaptureEnd;

	bool IsForeground;
	bool NeedsSeparateDepthRead;

	bool WasFrameRequested = false;
	ExpFrame RequestedFrame;
	FSceneView* CurView;

#if MIXCAST_SHADER_PASSES
	ExpFrame RenderingFrame;

	//Opaque+depth layer
	UTextureRenderTarget2D* OpaqueTarget = NULL;
	UTextureRenderTarget2D* ProcessedOpaqueColorTarget = NULL;

	//Foreground occlusion
	TSharedPtr<FMixCastSceneViewExt, ESPMode::ThreadSafe> viewExt;
	TMap<void*, WatchedUnrealSharedTexture*> sharedOcclTexs;

	WatchedUnrealSharedTexture* GetSharedOcclusionTex(void* sharedHandle, unsigned int expectedWidth, unsigned int expectedHeight);
#endif

	//Alpha channel
	UTextureRenderTarget2D* AlphaTarget = NULL;



	void RegisterRenderEvents();
	void UnregisterRenderEvents();

#if MIXCAST_SHADER_PASSES
	//PostOpaque
	FPostOpaqueRenderDelegate postOpaqueDel;
#if UE_4_26_OR_LATER
	FDelegateHandle postOpaqueDelHandle;
#endif
	void OnPostOpaque_RenderThread(FPostOpaqueRenderParameters& Parameters);
#endif

	//PostOverlay
	FPostOpaqueRenderDelegate postOverlayDel;
#if UE_4_26_OR_LATER
	FDelegateHandle postOverlayDelHandle;
#endif
	void OnPostOverlay_RenderThread(FPostOpaqueRenderParameters& Parameters);

	//PostResolve
	TBaseDelegate<void, FRHICommandListImmediate& /*RHICmdList*/, class FSceneRenderTargets& /*SceneContext*/> postResolveDel;
	FDelegateHandle postResolveDelHandle;
	void OnPostResolve_RenderThread(FRHICommandListImmediate& RHICmdList, class FSceneRenderTargets& SceneContext);

public:
	void Initialize(bool IsForeground);

	void BeginPlay() override;
	void EndPlay(EEndPlayReason::Type EndPlayReason) override;

	void AssignLayersTarget(UTextureRenderTarget2D* NewColorTarget, 
		UTextureRenderTarget2D* NewAlphaTarget);
#if MIXCAST_SHADER_PASSES
	void AssignOpaqueTargets(UTextureRenderTarget2D* NewOpaqueColorTarget, UTextureRenderTarget2D* NewProcessedOpaqueColorTarget);
#endif

	void CaptureFrameDeferred(ExpFrame frame);

	virtual void UpdateSceneCaptureContents(FSceneInterface* Scene) override;

	FCaptureEvent& OnCaptureBegin() { return CaptureBegin; }
	FCaptureEvent& OnCaptureEnd() { return CaptureEnd; }
};
