/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "CoreMinimal.h"
#include "Engine/CanvasRenderTarget2D.h"
#include "Materials/MaterialInstanceDynamic.h"

#include "MixCastSliceCanvas.generated.h"

class AExpCamera;

UENUM()
namespace EMaterialAlphaType
{
	enum Type
	{
		Opaque = 0,
		Keyed = 1,
		ForceAdditive = 2,
		AlphaChannel = 3,
		SeparateAlphaChannel = 4,
	};
}

UCLASS()
class MIXCAST_API UMixCastSliceCanvas : public UCanvasRenderTarget2D
{
	GENERATED_UCLASS_BODY()

public:
	UMixCastSliceCanvas();
	void Initialize(int width, int height, bool separatedAlpha);

	int GetLayerWidth() { return LayerWidth; }
	int GetLayerHeight() { return LayerHeight; }

	UTextureRenderTarget2D* GetFullTex() { return fullRenderTex; }
	UTextureRenderTarget2D* GetFullAlphaTex() { return fullAlphaTex; }
	UTextureRenderTarget2D* GetForegroundTex() { return foregroundTex; }
	UTextureRenderTarget2D* GetForegroundAlphaTex() { return foregroundAlphaTex; }

	//'arguments' for blit
	bool blitFull;
	bool blitForeground;

	bool renderTranslucent; 

protected:
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Render Targets")
		UTextureRenderTarget2D* fullRenderTex;
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Render Targets")
		UTextureRenderTarget2D* fullAlphaTex;
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Render Targets")
		UTextureRenderTarget2D* foregroundTex;	
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Render Targets")
		UTextureRenderTarget2D* foregroundAlphaTex;

	UFUNCTION()
	void BlitLayers(UCanvas* Canvas, int32 Width, int32 Height);

	int LayerWidth, LayerHeight;
	bool SeparateAlpha;

	UTextureRenderTarget2D* CreateRenderTarget(const TCHAR* name, int width, int height);

private:
	UPROPERTY(VisibleAnywhere, Category = "Materials")
		TArray<UMaterialInstanceDynamic*> fullRenderMats;
	UPROPERTY(VisibleAnywhere, Category = "Materials")
		TArray<UMaterialInstanceDynamic*> foregroundMats;

	UMaterialInstanceDynamic* GetTransferMaterial(TArray<UMaterialInstanceDynamic*>& mats,
		bool isTransparent, bool separatedAlpha);
};