#pragma once

#if PLATFORM_WINDOWS
#include "Windows/AllowWindowsPlatformTypes.h"
#endif
#include <Windows.h>
#include <stdio.h>
#include <conio.h>
#if PLATFORM_WINDOWS
#include "Windows/HideWindowsPlatformTypes.h"
#endif

#include "SharedQueueWriter.hpp"

struct SharedQueueProducerDesc
{
	int capacity;
};

template <class T>
class SharedQueueProducer
{
private:
	int capacity = 0;

	HANDLE descFile = NULL;
	SharedQueueWriter<T>** elements;

	unsigned long producedCount = 0;

public:
	int GetCurrentSlot()
	{
		return producedCount % capacity;
	}

private:
	SharedQueueWriter<T>* GetCurrentWriter()
	{
		return elements[GetCurrentSlot()];
	}

public:
	SharedQueueProducer(LPCWSTR chainId, int capacity) {

		this->capacity = capacity;
		int descSize = sizeof(SharedQueueProducerDesc);

		LPWSTR key = new TCHAR[256];
		//Metadata
		wsprintf(key, TEXT("MixCast-Chain(%s)-Desc"), chainId);
		descFile = CreateFileMapping(
			INVALID_HANDLE_VALUE,		// use paging file
			NULL,						// default security
			PAGE_READWRITE,				// read/write access
			0,							// maximum object size (high-order DWORD)
			descSize,					// maximum object size (low-order DWORD)
			key);						// name of mapping object
		SharedQueueProducerDesc* sharedDesc = (SharedQueueProducerDesc*)MapViewOfFile(descFile,   // handle to map object
			FILE_MAP_ALL_ACCESS, // read/write permission
			0,
			0,
			descSize);
		sharedDesc->capacity = capacity;
		UnmapViewOfFile(sharedDesc);

		elements = new SharedQueueWriter<T>*[capacity];
		for (int i = 0; i < capacity; i++)
		{
			wsprintf(key, TEXT("%s[%d]"), chainId, i);
			elements[i] = new SharedQueueWriter<T>(key);
		}

		delete[] key;
	}
	~SharedQueueProducer()
	{
		for (int i = 0; i < capacity; i++) {
			elements[i]->BeginDestroy();
			elements[i] = NULL;
		}
		delete[] elements;

		CloseHandle(descFile);
	}

	void EnqueueFenceUpdates()
	{
		for (int i = 0; i < capacity; i++)
			elements[i]->EnqueueFenceUpdate();
	}

	bool WaitUntilNextEmptied(int timeout = 0)
	{
		return GetCurrentWriter()->WaitUntilEmptied(timeout);
	}
	void Write(T data, bool requiresGpuOp)
	{
		GetCurrentWriter()->Write(data, requiresGpuOp);
		producedCount++;
	}
};

