#pragma once

#include "WatchedUnrealSharedTexture_D3D11.h"
#include "../../../MixCastLog.h"

#if PLATFORM_WINDOWS
#include "Windows/AllowWindowsPlatformTypes.h"
#endif
#include <d3d11_1.h>
#include <dxgi1_2.h>
#if PLATFORM_WINDOWS
#include "Windows/HideWindowsPlatformTypes.h"
#endif

#if MIXCAST_D3D12
#include "ITextureShareD3D11.h"
#endif

void WatchedUnrealSharedTexture_D3D11::Initialize(HANDLE texHandle)
{
	device = (ID3D11Device1*)GDynamicRHI->RHIGetNativeDevice();

	IDXGIResource* dxgi_res;
	HRESULT hr;
	if (ntHandles)
		hr = device->OpenSharedResource1(texHandle, IID_PPV_ARGS(&dxgi_res));
	else
		hr = device->OpenSharedResource(texHandle, IID_PPV_ARGS(&dxgi_res));

	if (FAILED(hr))
		return;

	hr = dxgi_res->QueryInterface(__uuidof(ID3D11Texture2D), (void**)&sharedTex);
	if (FAILED(hr))
		return;

	D3D11_TEXTURE2D_DESC refDesc;
	sharedTex->GetDesc(&refDesc);

	if (FAILED(hr))
		return;

	width = refDesc.Width;
	height = refDesc.Height;

	if (ntHandles)
		dxgi_res->QueryInterface<IDXGIKeyedMutex>(&mutex);

	dxgi_res->Release();

#if MIXCAST_D3D12
	if (!ITextureShareD3D11::Get().CreateRHITexture(sharedTex, GetUnrealFormatFromDirectXFormat(refDesc.Format), sharedTexRhi))
		MIXCAST_ERROR("Couldn't create RHI for shared watched texture");
#endif

	dstTex = UTexture2D::CreateTransient(refDesc.Width, refDesc.Height, GetUnrealFormatFromDirectXFormat(refDesc.Format));
	dstTex->AddToRoot();
	dstTex->UpdateResource();
}

void WatchedUnrealSharedTexture_D3D11::BeginDestroy()
{
	SAFE_RELEASE(sharedTex);
	if (ntHandles) {
		SAFE_RELEASE(mutex);
		CloseHandle(localHandle);
	}

	if (dstTex && dstTex->IsValidLowLevel())
	{
		if (dstTex->Resource)
			dstTex->ReleaseResource();
		dstTex->RemoveFromRoot();
		dstTex->MarkPendingKill();
	}
	dstTex = nullptr;

	ENQUEUE_RENDER_COMMAND(void)(
		[&](FRHICommandListImmediate& RHICmdList)
		{
			delete this;
		});
}

void WatchedUnrealSharedTexture_D3D11::Read_RenderThread(FRHICommandListImmediate& RHICmdList)
{
	if (dstTex == NULL || sharedTex == NULL)
		return;

#if MIXCAST_D3D12
	if (!dstTex->IsValidLowLevel())
		return;
#else
	ID3D11Texture2D* dst = (ID3D11Texture2D*)GetNativeTexture(dstTex);
	if (dst == NULL)
		return;
#endif

	if (mutex && !SUCCEEDED(mutex->AcquireSync(0, INFINITE)))
		return;

#if MIXCAST_D3D12
	FBox2D CopyBox(FVector2D(0.0f, 0.0f), FVector2D(dstTex->GetSurfaceWidth(), dstTex->GetSurfaceHeight()));
	GDynamicRHI->RHICopySubTextureRegion(sharedTexRhi, dstTex->Resource->GetTexture2DRHI(), CopyBox, CopyBox);
#else
	ID3D11DeviceContext* ctx = NULL;
	device->GetImmediateContext(&ctx);
	ctx->CopyResource(dst, sharedTex);
	ctx->Release();
#endif

	if (mutex)
		mutex->ReleaseSync(0);
}