/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include <chrono>
#include <string>

#if PLATFORM_WINDOWS
#include "Windows/AllowWindowsPlatformTypes.h"
#endif

#include "CoreMinimal.h"
#include "Containers/UnrealString.h"
#include "Misc/App.h"
#include "Misc/ConfigCacheIni.h"
#include "Engine/Engine.h"
#include "Engine/GameViewportClient.h"
#include "Windows/WindowsPlatformProcess.h"
#include "Windows/WindowsPlatformMisc.h"

class MIXCAST_API TimeUtils
{
public:
	static const unsigned long TicksPerSecond = 1000 * 10000;

	static unsigned long long GetMixCastTimestamp()
	{
		return std::chrono::system_clock::now().time_since_epoch().count();
	}

	static FString GetSMPTE(unsigned long timestamp, float framerate)
	{
		unsigned long hoursPart = timestamp / (unsigned long)(framerate * 60 * 60);
		unsigned long minutesPart = (timestamp / (unsigned long)(framerate * 60)) % 60;
		unsigned long secondsPart = (timestamp / (unsigned long)framerate) % 60;
		unsigned long framePart = timestamp % (unsigned long)framerate;
		return FString::Printf(TEXT("%02d:%02d:%02d:%02d"), (int)hoursPart, (int)minutesPart, (int)secondsPart, (int)framePart); //assumes framerate is less than 100
	}

	static unsigned long GetFrameIndexForTimestamp(unsigned long long durationTicks, float frameRate)
	{
		unsigned long long extraDuration = (unsigned long long)(TicksPerSecond / 2 / frameRate);
		return (unsigned long)((double)frameRate * (durationTicks + extraDuration) / TicksPerSecond);
	}

	static unsigned long GetMidnightRelativeFrameIndex(unsigned long long timestamp, float frameRate)
	{
		const unsigned long long TicksPerDay = 24LL * 60 * 60 * 1000 * 10000;
		unsigned long long ticksPerFrame = (unsigned long long)(TicksPerSecond / (double)frameRate);
		return GetFrameIndexForTimestamp((timestamp - timestamp % ticksPerFrame) % TicksPerDay, frameRate);
	}
};

#if PLATFORM_WINDOWS
#include "Windows/HideWindowsPlatformTypes.h"
#endif