/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "ReadDepth.h"
#if MIXCAST_SHADER_PASSES
#include "CommonRenderResources.h"
#include "ScreenRendering.h"
#include "Runtime/Renderer/Private/SceneRendering.h"
#include "Runtime/Renderer/Private/PostProcess/SceneRenderTargets.h"

class FReadDepthPS : public FGlobalShader
{
public:
	DECLARE_GLOBAL_SHADER(FReadDepthPS);

	static bool ShouldCompilePermutation(const FGlobalShaderPermutationParameters& Parameters)
	{
		return IsFeatureLevelSupported(Parameters.Platform, ERHIFeatureLevel::SM5);
	}

	static void ModifyCompilationEnvironment(const FGlobalShaderPermutationParameters& Parameters, FShaderCompilerEnvironment& OutEnvironment)
	{
		FGlobalShader::ModifyCompilationEnvironment(Parameters, OutEnvironment);
	}

protected:
#if UE_4_25_OR_LATER
	LAYOUT_FIELD(FShaderResourceParameter, DepthTexture);
	LAYOUT_FIELD(FShaderResourceParameter, DepthTextureSampler);
#else
	FShaderResourceParameter DepthTexture;
	FShaderResourceParameter DepthTextureSampler;
#endif

public:
	FReadDepthPS() { }
	FReadDepthPS(const ShaderMetaType::CompiledShaderInitializerType& Initializer)
		: FGlobalShader(Initializer)
	{
		DepthTexture.Bind(Initializer.ParameterMap, TEXT("DepthTexture"), SPF_Mandatory);
		DepthTextureSampler.Bind(Initializer.ParameterMap, TEXT("DepthTextureSampler"));
	}

#if !UE_4_25_OR_LATER
	virtual bool Serialize(FArchive& Ar) override
	{
		bool bShaderHasOutdatedParameters = FGlobalShader::Serialize(Ar);
		Ar << DepthTexture << DepthTextureSampler;
		return bShaderHasOutdatedParameters;
	}
#endif

	void SetParameters(FRHICommandList& RHICmdList, TShaderMapRef<FReadDepthPS> ShaderRef, FRHISamplerState* SamplerStateRHI, FRHITexture* TextureRHI)
	{
#if !UE_4_25_OR_LATER
		FRHIPixelShader* ShaderRHI = GetPixelShader();
#else
		FRHIPixelShader* ShaderRHI = ShaderRef.GetPixelShader();
#endif

		SetTextureParameter(RHICmdList, ShaderRHI, DepthTexture, DepthTextureSampler, SamplerStateRHI, TextureRHI);
	}

	// void SetParameters(FRHICommandList& RHICmdList, TShaderMapRef<FReadDepthPS> ShaderRef, FSceneView& InView, FRHITexture* DepthTextureRHI)
	// {
	//	 FRHISamplerState* SamplerStateRHI = TStaticSamplerState<SF_Point>::GetRHI();
 //#if !UE_4_25_OR_LATER
	//	 FRHIPixelShader* ShaderRHI = GetPixelShader();
 //#else
	//	 FRHIPixelShader* ShaderRHI = ShaderRef.GetPixelShader();
 //#endif

	//	 //FGlobalShader::SetParameters<FViewUniformShaderParameters>(RHICmdList, ShaderRHI, InView.ViewUniformBuffer); 
	//	 //SetTextureParameter(RHICmdList, ShaderRHI, DepthTexture, DepthTextureSampler, SamplerStateRHI, DepthTextureRHI);

	//	 SetTextureParameter(RHICmdList, RHICmdList.GetBoundPixelShader(), DepthTexture, DepthTextureSampler, SamplerStateRHI, DepthTextureRHI);
	// }
};

IMPLEMENT_GLOBAL_SHADER(FReadDepthPS, "/Plugin/MixCast/ReadDepth.usf", "MainPS", SF_Pixel);


void FMixCastShader_ReadDepth::Execute(IRendererModule& RendererModule, FRHICommandList& RHICmdList, FSceneView& InView, UTextureRenderTarget2D& OpaqueLayerTarget)
{
	const uint32 surfaceWidth = OpaqueLayerTarget.GetSurfaceWidth();
	const uint32 surfaceHeight = OpaqueLayerTarget.GetSurfaceHeight();

	FSceneRenderTargets& SceneContext = FSceneRenderTargets::Get(RHICmdList);

	FGraphicsPipelineStateInitializer GraphicsPSOInit;
	GraphicsPSOInit.RasterizerState = TStaticRasterizerState<FM_Solid, CM_None>::GetRHI();
	GraphicsPSOInit.DepthStencilState = TStaticDepthStencilState<false, CF_Always>::GetRHI();
	GraphicsPSOInit.BlendState = TStaticBlendState<CW_ALPHA>::GetRHI();


#if UE_4_22_OR_LATER
	FRHIRenderPassInfo RPInfo(OpaqueLayerTarget.GetRenderTargetResource()->GetRenderTargetTexture(), ERenderTargetActions::DontLoad_Store);
	RHICmdList.BeginRenderPass(RPInfo, TEXT("MixCast-ReadDepth"));
#else
	FRHIRenderTargetView TargetView(OpaqueLayerTarget.GetRenderTargetResource()->GetRenderTargetTexture(), 0, -1, ERenderTargetLoadAction::ENoAction, ERenderTargetStoreAction::EStore);
	FRHIDepthRenderTargetView DepthView;
	RHICmdList.SetRenderTargets(1, &TargetView, &DepthView, 0, NULL);
#endif

	RHICmdList.ApplyCachedRenderTargets(GraphicsPSOInit);
	RHICmdList.SetViewport(0, 0, 0, surfaceWidth, surfaceHeight, 1);

	auto ShaderMap = GetGlobalShaderMap(GMaxRHIFeatureLevel);
	TShaderMapRef<FScreenVS> VertexShader(ShaderMap);
	TShaderMapRef<FReadDepthPS> PixelShader(ShaderMap);

#if UE_4_22_OR_LATER
	GraphicsPSOInit.BoundShaderState.VertexDeclarationRHI = GFilterVertexDeclaration.VertexDeclarationRHI;
#else
	GraphicsPSOInit.BoundShaderState.VertexDeclarationRHI = RendererModule.GetFilterVertexDeclaration().VertexDeclarationRHI;
#endif

#if UE_4_25_OR_LATER
	GraphicsPSOInit.BoundShaderState.VertexShaderRHI = VertexShader.GetVertexShader();
	GraphicsPSOInit.BoundShaderState.PixelShaderRHI = PixelShader.GetPixelShader();
#else
	GraphicsPSOInit.BoundShaderState.VertexShaderRHI = GETSAFERHISHADER_VERTEX(*VertexShader);
	GraphicsPSOInit.BoundShaderState.PixelShaderRHI = GETSAFERHISHADER_PIXEL(*PixelShader);
#endif

	GraphicsPSOInit.PrimitiveType = PT_TriangleList;
	SetGraphicsPipelineState(RHICmdList, GraphicsPSOInit);


	VertexShader->SetParameters(RHICmdList, InView.ViewUniformBuffer);
	PixelShader->SetParameters(RHICmdList, PixelShader, TStaticSamplerState<SF_Point>::GetRHI(), SceneContext.GetSceneDepthTexture());

#if UE_4_19_OR_LATER
	const FViewInfo* ActualView = (FViewInfo*)(&InView);
#else
	const FSceneView* ActualView = &SceneView;
#endif

	RendererModule.DrawRectangle(
		RHICmdList,
		ActualView->ViewRect.Min.X, ActualView->ViewRect.Min.Y,
		ActualView->ViewRect.Width(), ActualView->ViewRect.Height(),
		ActualView->ViewRect.Min.X, ActualView->ViewRect.Min.Y,
		ActualView->ViewRect.Width(), ActualView->ViewRect.Height(),
		InView.UnconstrainedViewRect.Size(),
		SceneContext.GetBufferSizeXY(),
#if UE_4_25_OR_LATER
		VertexShader,
#else
		*VertexShader,
#endif
		EDRF_UseTriangleOptimization);

#if UE_4_22_OR_LATER
	RHICmdList.EndRenderPass();
#endif
}
#endif